@@ -15,7 +15,7 @@ use warnings;
 use parent qw( Exporter );
 use HiPi::RaspberryPi;
 
-our $VERSION ='0.61';
+our $VERSION ='0.62';
 
 our @EXPORT_OK = ( qw( hipi_export_ok  hipi_export_constants hipi_export_tags ) );
 our %EXPORT_TAGS = ( hipi => \@EXPORT_OK );
@@ -731,6 +731,12 @@ my $const = {
     },
     
     openthings => {
+        
+        OPENTHINGS_MANUFACTURER_ENERGENIE   => 0x04,
+        OPENTHINGS_MANUFACTURER_SENTEC      => 0x01,
+        OPENTHINGS_MANUFACTURER_HILDERBRAND => 0x02,
+        OPENTHINGS_MANUFACTURER_RASPBERRY   => 0x3F,
+        
         OPENTHINGS_PARAM_ALARM           => 0x21,
         OPENTHINGS_PARAM_DEBUG_OUTPUT    => 0x2D,
         OPENTHINGS_PARAM_IDENTIFY        => 0x3F,
@@ -784,29 +790,33 @@ my $const = {
         OPENTHINGS_PARAM_TEST            => 0xAA,
         OPENTHINGS_WRITE_MASK            => 0x80,
         
-        OPENTHINGS_UINT      => 0x00,
-        OPENTHINGS_UINT_BP4  => 0x10,
-        OPENTHINGS_UINT_BP8  => 0x20,
-        OPENTHINGS_UINT_BP12 => 0x30,
-        OPENTHINGS_UINT_BP16 => 0x40,
-        OPENTHINGS_UINT_BP20 => 0x50,
-        OPENTHINGS_UINT_BP24 => 0x60,
-        OPENTHINGS_CHAR      => 0x70,
-        OPENTHINGS_SINT      => 0x80,
-        OPENTHINGS_SINT_BP8  => 0x90,
-        OPENTHINGS_SINT_BP16 => 0xA0,
-        OPENTHINGS_SINT_BP24 => 0xB0,
-        # C0,D0,E0 RESERVED
-        OPENTHINGS_FLOAT     => 0xF0,
+        OPENTHINGS_UINT        => 0x00,
+        OPENTHINGS_UINT_BP4    => 0x10,
+        OPENTHINGS_UINT_BP8    => 0x20,
+        OPENTHINGS_UINT_BP12   => 0x30,
+        OPENTHINGS_UINT_BP16   => 0x40,
+        OPENTHINGS_UINT_BP20   => 0x50,
+        OPENTHINGS_UINT_BP24   => 0x60,
+        OPENTHINGS_CHAR        => 0x70,
+        OPENTHINGS_SINT        => 0x80,
+        OPENTHINGS_SINT_BP8    => 0x90,
+        OPENTHINGS_SINT_BP16   => 0xA0,
+        OPENTHINGS_SINT_BP24   => 0xB0,
+        OPENTHINGS_ENUMERATION => 0xC0,
+        # D0,E0 RESERVED
+        OPENTHINGS_FLOAT       => 0xF0,
     },
     
     energenie => {
+        ENERGENIE_ENER314_DUMMY_GROUP => 0xFFFFFF,
+        
         ENERGENIE_MANUFACTURER_ID    => 0x04,
     
         ENERGENIE_PRODUCT_ID_MIHO004 => 0x01,
         ENERGENIE_PRODUCT_ID_MIHO005 => 0x02,
-        ENERGENIE_PRODUCT_ID_MIHO006 => 0x05,
         ENERGENIE_PRODUCT_ID_MIHO013 => 0x03,
+        ENERGENIE_PRODUCT_ID_MIHO006 => 0x05,
+        
         ENERGENIE_PRODUCT_ID_MIHO032 => 0x0C,
         ENERGENIE_PRODUCT_ID_MIHO033 => 0x0D,
         
@@ -814,8 +824,8 @@ my $const = {
         ENERGENIE_DEFAULT_CRYPTPIP   => 0x0100,
         
         ENERGENIE_FIFOTHRESH_FSK    => 0x81, # Condition to start packet transmission: at least one byte in FIFO
-        ENERGENIE_FIFOTHRESH_OOK    => 0x1E,  # Condition to start packet transmission: wait for 30 bytes in FIFO
-        ENERGENIE_TXOOK_REPEAT_RATE => 15,
+        ENERGENIE_FIFOTHRESH_OOK    => 0x1E, # Condition to start packet transmission: wait for 30 bytes in FIFO
+        ENERGENIE_TXOOK_REPEAT_RATE => 25,
         ENERGENIE_MESSAGE_BUF_SIZE  => 66,
         ENERGENIE_MAX_FIFO_SIZE     => 66,
         ENERGENIE_NODEADDRESS01	 => 0x01,  # Node address used in address filtering
@@ -842,7 +852,7 @@ my $const = {
         ENERGENIE_PACKETCONFIG1_OOK	      => 0,  # Fixed length, no Manchester coding
         ENERGENIE_NODEADDRESS                => 0x06, # Node address used in address filtering ( when enabled )
         
-        ENERGENIE_PAYLOADLEN_OOK	=> 13 + 8 * 17,  # fixed OOK Payload Length  
+        ENERGENIE_PAYLOADLEN_OOK	=> 13 + 8 * 17,  # fixed OOK Payload Length
     },
     
     si470n => {
