@@ -21,10 +21,10 @@ use HiPi::GPIO;
 use HiPi::RF::OpenThings::Message;
 
 __PACKAGE__->create_accessors( qw( device devicename led_green led_red 
-                              led_on _green_pin _red_pin repeat backend
-                              default_config ook_config gpiodev) );
+                              led_on _green_pin _red_pin ook_repeat
+                              backend default_config ook_config gpiodev) );
 
-our $VERSION ='0.61';
+our $VERSION ='0.62';
 
 sub new {
     my( $class, %userparams ) = @_;
@@ -40,7 +40,7 @@ sub new {
         led_red        => RPI_PIN_15,
         led_on         => 1,
         reset_gpio     => RPI_PIN_22,
-        repeat         => ENERGENIE_TXOOK_REPEAT_RATE,
+        oo_repeat     => ENERGENIE_TXOOK_REPEAT_RATE,
         gpiodev        => HiPi::GPIO->new,
         default_config => [
             [ RF69_REG_REGDATAMODUL,  0x00 ],   # modulation scheme FSK
@@ -101,7 +101,7 @@ sub new {
             delay        => $params{delay},
             devicename   => $params{devicename},
             reset_gpio   => $params{reset_gpio},
-            repeat       => $params{repeat},
+            ook_repeat   => $params{ook_repeat},
             backend      => $params{backend},
             fsk_config   => $params{default_config},
             ook_config   => $params{ook_config},
@@ -150,7 +150,7 @@ sub send_message {
 sub send_ook_message {
     my($self, $groupid, $data, $repeat ) = @_;
     
-    $repeat ||= $self->repeat;
+    $repeat ||= $self->ook_repeat;
         
     $self->set_red_led( 1 );
     
@@ -226,7 +226,7 @@ sub receive_fsk_message {
             databuffer => $buffer,
             cryptseed  => $encryptionid,
         );
-        $msg->decode_buffer;
+        $msg->inspect_buffer;
         return $msg;
     }
     return undef;
@@ -234,29 +234,10 @@ sub receive_fsk_message {
 
 sub send_fsk_message {
     my ($self, $msg) =  @_;
-    $msg->encode_buffer unless $msg->buffer_length;
+    $msg->encode_buffer unless $msg->is_encoded;
     $self->send_message( $msg->databuffer );
 }
 
-sub map_product_to_name {
-    my($self, $productid) = @_;
-    my $name = 'Unknown Product';
-    if( $productid == ENERGENIE_PRODUCT_ID_MIHO004 ) {
-        $name = 'Mi|Home Monitor';
-    } elsif( $productid == ENERGENIE_PRODUCT_ID_MIHO005 ) {
-        $name = 'Mi|Home Adapter Plus';
-    } elsif( $productid == ENERGENIE_PRODUCT_ID_MIHO006 ) {
-        $name = 'Mi|Home Whole House Monitor';
-    } elsif( $productid == ENERGENIE_PRODUCT_ID_MIHO013 ) {
-        $name = 'Mi|Home Heating Valve';
-    } elsif( $productid == ENERGENIE_PRODUCT_ID_MIHO032 ) {
-        $name = 'Mi|Home Motion Sensor';
-    } elsif( $productid == ENERGENIE_PRODUCT_ID_MIHO033 ) {
-        $name = 'Mi|Home Open Sensor';
-    }
-    return $name;
-}
-
 #-------------------------------------------------------
 # Common OOK switch handler
 #-------------------------------------------------------
