@@ -13,10 +13,12 @@ package HiPi::RF::OpenThings::Message;
 use strict;
 use warnings;
 use parent qw( HiPi::Class );
-use HiPi qw( :openthings );
+use HiPi qw( :openthings :energenie );
+use HiPi::RF::OpenThings;
 use Try::Tiny;
+use JSON;
 
-our $VERSION ='0.61';
+our $VERSION ='0.62';
 
 __PACKAGE__->create_accessors( qw(
     cryptseed
@@ -25,23 +27,41 @@ __PACKAGE__->create_accessors( qw(
     epoch
     length
     ok
+    configured_name
     records
     _mid
     _pid
     _sid
     _pip
+    is_decoded
+    is_encoded
+    has_join_cmd
+    has_join_ack
+    switch_state
+    switch_command
+    has_command
 ));
 
 sub new {
     my( $class, %params ) = @_;
+    
+    my( $sk_mid, $sk_pid, $sk_sid ) = ( 0, 0, 0 );
+    
+    if($params{sensor_key}) {
+        ( $sk_mid, $sk_pid, $sk_sid ) = split(/-/, $params{sensor_key} );
+        for ( $sk_mid, $sk_pid, $sk_sid ) {
+            $_ = hex($_);
+        }
+    }
+    
     $params{epoch} = time();
     $params{errorbuffer}  = [];
     $params{databuffer} //= [];
     $params{cryptseed} //= 1;
     $params{records} = [];
-    $params{_mid} = $params{mid} || $params{manufacturer_id} || 0;
-    $params{_pid} = $params{pid} || $params{product_id} || 0;
-    $params{_sid} = $params{sid} || $params{sensor_id} || 0;
+    $params{_mid} = $sk_mid || $params{mid} || $params{manufacturer_id} || 0;
+    $params{_pid} = $sk_pid || $params{pid} || $params{product_id} || 0;
+    $params{_sid} = $sk_sid || $params{sid} || $params{sensor_id} || 0;
     $params{_pip} = $params{pip} || $params{encrypt_pip} || 0;
     my $self = $class->SUPER::new( %params );
     return $self;
@@ -55,10 +75,84 @@ sub sensor_id { return $_[0]->_sid; }
 
 sub encrypt_pip { return $_[0]->_pip; }
 
+sub has_switch_state {
+    my $self = shift;
+    return ( defined($self->switch_state) ) ? 1 : 0;
+}
+
 sub sensor_key {
     my $self = shift;
-    my $key = sprintf(qq(%04X-%04X-%06X), $self->manufacturer_id || 0, $self->product_id || 0, $self->sensor_id || 0);
-    return $key;    
+    return HiPi::RF::OpenThings->format_sensor_key($self->manufacturer_id, $self->product_id, $self->sensor_id);  
+}
+
+sub product_name {
+    my( $self  ) = @_;
+    return HiPi::RF::OpenThings->product_name($self->manufacturer_id, $self->product_id );
+}
+
+sub manufacturer_name {
+    my( $self  ) = @_;
+    return HiPi::RF::OpenThings->manufacturer_name($self->manufacturer_id );
+}
+
+sub value_hash {
+    my $self = shift;
+    $self->decode_buffer unless $self->is_decoded;
+    
+    my $data = {
+        timestamp       => $self->timestamp,
+        manufacturer_id => $self->manufacturer_id,
+        product_id      => $self->product_id,
+        product_name    => $self->product_name,
+        sensor_id       => $self->sensor_id,
+        sensor_key      => $self->sensor_key,
+        records         => [],
+        manufacturer_name => $self->manufacturer_name,
+        configured_name => $self->configured_name || '',
+    };
+    
+    for my $record ( @{ $self->records } ) {
+        push(  @{ $data->{records} },
+             {
+                name    => $record->name,
+                value   => $record->value,
+                units   => $record->units,
+                command => $record->command,
+                id      => $record->id,
+                value_type => $record->typeid,
+             }
+        );
+        
+        if( $record->id == OPENTHINGS_PARAM_JOIN ) {
+            if( $record->command ) {
+                $data->{join_command} = 1;
+            } else {
+                $data->{join_ack} = 1;
+            }
+        }
+        
+        if( $record->id == OPENTHINGS_PARAM_SWITCH_STATE ) {
+            $data->{switch_state} = $record->value;
+            if( $record->command ) {
+                $data->{switch_command} = 1;
+            }
+        }
+    }
+    return $data;
+}
+
+sub json {
+    my ($self, $pretty) = @_;
+    my $data = $self->value_hash;
+    my $output = try {
+        my $j = JSON->new;
+        my $return = ( $pretty ) ? $j->pretty->canonical->encode( $data ) : $j->encode( $data );
+        return $return;
+    } catch {
+        my $error = $_;
+        $error =~ s/[\n"']+/ /g;
+        return qq({"ok":0,"error":"$error"});
+    };
 }
 
 sub encode_buffer {
@@ -74,21 +168,42 @@ sub encode_buffer {
         push @$payload, $self->sensor_id  & 0xFF;
         
         for my $record ( @{ $self->records } ) {
-            my $writemask = ( $record->wr ) ? OPENTHINGS_WRITE_MASK : 0x0;
+            # get some convenience values
+            
+            $self->has_command(1) if $record->command;
+            
+            if( $record->id == OPENTHINGS_PARAM_JOIN  ) {    
+                if( $record->command ) {
+                    $self->has_join_cmd(1);
+                } else {
+                    $self->has_join_ack(1);
+                }
+                
+            } elsif($record->id == OPENTHINGS_PARAM_SWITCH_STATE  ) {
+                $self->switch_state( $record->value );
+                if( $record->command ) {
+                    $self->switch_command(1);
+                }
+            }
+                        
+            my $writemask = ( $record->command ) ? OPENTHINGS_WRITE_MASK : 0x0;
             # ID and R/W
             push @$payload, $record->id | $writemask;
-            # Type (will and length later)
+            # Type (will or/| length later)
             push @$payload, $record->typeid & 0xF0;
+            
             # record position of type/length byte
             my $lenpos = ( scalar @$payload ) -1;
             
             my $val = $record->value;
             if(defined($val) && $val ne '') {
-                my @valbytes = $self->encode_value( $record->typeid, $val, $record->length );
-                my $vallen = ( @valbytes ) ? scalar(@valbytes) : 0;
+                my @valbytes = $self->encode_value( $record->typeid, $val );
+                my $vallen = scalar( @valbytes );
+                # max record length is 15 bytes;
+                $vallen = 15 if $vallen > 15;
                 $payload->[$lenpos] = ( $record->typeid & 0xF0 ) | ( $vallen & 0xF );
-                for my $byte ( @valbytes ) {
-                    push @$payload, $byte;
+                for (my $vindex = 0; $vindex < $vallen; $vindex ++ ) {
+                    push @$payload, $valbytes[$vindex];
                 }
             }
         }
@@ -112,10 +227,39 @@ sub encode_buffer {
         $self->push_error(q(unexpected error in message encode ) . $_);
         return 0;
     };
-    
+    $self->is_encoded(1);
     $self->ok( $result );
 }
 
+
+sub inspect_buffer {
+    my $self = shift;
+    my $result = try {
+        
+        # check basic message length
+        {
+            my $bytelength = $self->buffer_length;
+            
+            if ( ( $bytelength < 11 )
+                || ( $self->databuffer->[0] + 1 != $bytelength ) ) {
+                $self->push_error(q(invalid message length ) . $bytelength);
+                return 0;
+            }
+        }
+        
+        $self->_mid( $self->databuffer->[1] );
+        $self->_pid( $self->databuffer->[2] );
+        $self->_pip( ( $self->databuffer->[3] << 8 ) + $self->databuffer->[4] );
+        
+        return 1;
+        
+    } catch {
+        return 0;
+    };
+    
+    return $result;
+}
+
 sub decode_buffer {
     my $self = shift;
     my $result = try {
@@ -124,19 +268,27 @@ sub decode_buffer {
         $self->records([]);
         
         # check basic message length
-        if ( ( $self->buffer_length < 10 )
-            || ( $self->databuffer->[0] + 1 != $self->buffer_length ) ) {
-            $self->push_error(q(invalid message length ) . $self->buffer_length);
-            return 0;
+        {
+            my $bytelength = $self->buffer_length;
+            
+            if ( ( $bytelength < 11 )
+                || ( $self->databuffer->[0] + 1 != $bytelength ) ) {
+                $self->push_error(q(invalid message length ) . $bytelength);
+                return 0;
+            }
         }
         
-        $self->crypt_buffer;
-        
         my $payload = $self->databuffer;
         
         $self->_mid( $payload->[1] );
         $self->_pid( $payload->[2] );
         $self->_pip( ( $payload->[3] << 8 ) + $payload->[4] );
+        
+        # new we have the manufacturer and product id we can optionally
+        # decrypt the message
+        
+        $self->crypt_buffer;
+        
         $self->_sid( ( $payload->[5] << 16 ) + ( $payload->[6] << 8 ) + $payload->[7] );
         
         # check CRC to see if this is good message
@@ -153,23 +305,23 @@ sub decode_buffer {
         my @records;
         while ( ( $index  < @$payload -2 ) && $payload->[$index] != 0 ) {
             my $param = $payload->[$index];
-            my $wr = (($param & 0x80) == 0x80);
+            my $command = (( $param & OPENTHINGS_WRITE_MASK ) == OPENTHINGS_WRITE_MASK ) ? 1 : 0;
 		    my $paramid = $param & 0x7F;
-            my ( $paramname, $paramunit ) = $self->ot_param_map( $paramid );
+            my ( $paramname, $paramunit ) = HiPi::RF::OpenThings->parameter_map( $paramid );
             $index ++;
             my $typeid = $payload->[$index] & 0xF0;
             my $paramlen = $payload->[$index] & 0x0F;
             $index ++;
             
             my $record = {
-                wr     => $wr,
-                id     => $paramid,
-                name   => $paramname,
-                units  => $paramunit,
-                typeid => $typeid,
-                length => $paramlen,
-                value  => '',
-                bytes  => [],
+                command => $command,
+                id      => $paramid,
+                name    => $paramname,
+                units   => $paramunit,
+                typeid  => $typeid,
+                length  => $paramlen,
+                value   => '',
+                bytes   => [],
             };
             
             if ( $paramlen != 0 ) {
@@ -189,6 +341,28 @@ sub decode_buffer {
                 $record->{value} = $value;
                 $record->{bytes} = \@valuebytes;
             }
+            
+            # get some convenience values
+            
+            $self->has_command(1) if $record->{command};
+            
+            if( $record->{id} == OPENTHINGS_PARAM_JOIN  ) {
+                if( $record->{command} ) {
+                    $self->has_join_cmd(1);
+                } else {
+                    $self->has_join_ack(1);
+                }
+            }
+            
+            if( $record->{id} == OPENTHINGS_PARAM_SWITCH_STATE  ) {
+                if(defined($record->{value}) && $record->{value} ne '' ) {
+                    $self->switch_state( $record->{value} );
+                    if( $record->{command} ) {
+                        $self->switch_command(1);
+                    }
+                }
+            }
+            
             push @records, $record;   
         }
     
@@ -201,6 +375,7 @@ sub decode_buffer {
         return 0;
     };
     
+    $self->is_decoded(1);
     $self->ok( $result );
 }
 
@@ -261,6 +436,7 @@ sub decode_value {
     my($self, $typeid, $bytes) = @_;
     
     my $numbytes = scalar @$bytes;
+    return undef unless $numbytes;
     
     if ( $typeid <= OPENTHINGS_UINT_BP24 ) {
 		my $result = 0;
@@ -276,15 +452,20 @@ sub decode_value {
             return $result / ( 2 ** $self->get_value_type_bits( $typeid ) );
         }
     } elsif( $typeid == OPENTHINGS_CHAR ) {
-        my $result = join('', @$bytes);
+        my $numbytes = scalar @$bytes;
+        my $format = 'C*';
+        my $result = pack($format, @$bytes);
         return $result;
     } elsif( $typeid >= OPENTHINGS_SINT && $typeid <= OPENTHINGS_SINT_BP24 ) {     
         # decode unsigned int first
-		my $result = 0;
+		
+        my $result = 0;
+        
 		for (my $i = 0; $i < @$bytes; $i++) {
 			$result <<= 8;
 			$result += $bytes->[$i];
         }
+        
 		# turn to signed int based on high bit of MSB
 		# 2's comp is 1's comp plus 1
 		if(($bytes->[0] & 0x80) == 0x80) {
@@ -297,6 +478,8 @@ sub decode_value {
         } else {
             return $result / ( 2 ** $self->get_value_type_bits( $typeid ) );
         }
+        
+        
     } elsif( $typeid == OPENTHINGS_FLOAT ) {
         return "TODO_FLOAT_IEEE_754-2008";
     }
@@ -305,15 +488,24 @@ sub decode_value {
 }
 
 sub encode_value {
-    my($self, $typeid, $value, $vallength ) = @_;
-    $vallength ||= 0;
+    my($self, $typeid, $value ) = @_;
     
     my @result = ();
     my @emptyresult = ();
-    
-    if ( $typeid <= OPENTHINGS_UINT_BP24 ) {
-        # unsigned integer
-        if ( $typeid != OPENTHINGS_UINT ) {
+        
+    if( $typeid == OPENTHINGS_CHAR ) {
+        unless(defined($value) && $value ne '' ) {
+            return @emptyresult;
+        }
+        @result = unpack('C*', $value );
+
+    } elsif( $typeid == OPENTHINGS_FLOAT ) {
+        warn "TODO_FLOAT_IEEE_754-2008";
+        return @emptyresult;
+        
+    } elsif ( $typeid <= OPENTHINGS_SINT_BP24 ) {
+        # signed and unsigned integers can be packed the same
+        if ( ( $typeid != OPENTHINGS_UINT ) && ( $typeid != OPENTHINGS_SINT ) ) {
 			# pre-adjust for BP
 			if( int($value) != $value ) {
 				$value *= ( 2 ** $self->get_value_type_bits( $typeid ) ); # shifts float into int range using BP
@@ -323,93 +515,12 @@ sub encode_value {
         
         my $v = $value;
         unshift(@result, $v & 0xFF );
-        $v >>= 8;        
-		
+        $v >>= 8;
+        
         while( $v != 0 ) {
             unshift(@result, $v & 0xFF );
             $v >>= 8;
         }
-        
-        if( $vallength && $vallength > ( scalar @result) ) {
-            # left pad with zeros
-			my $numbytes = $vallength - ( scalar @result );
-            for (my $index = 0; $index < $numbytes; $index ++) {
-                unshift(@result, 0);
-            }
-        }
-        
-    } elsif( $typeid == OPENTHINGS_CHAR ) {
-        unless(defined($value) && $value ne '' ) {
-            return @emptyresult;
-        }
-        my @chars = split(//, $value);
-        my $numchars = scalar @chars;
-        return @emptyresult unless $numchars;
-        $numchars = $vallength if( $vallength && $vallength < $numchars );
-        my $extrachars = (  $vallength && $vallength > $numchars ) ? $vallength - $numchars : 0;
-        
-        for (my $index = 0; $index < $numchars; $index ++) {
-            push @result, ord( $chars[$index]);
-        }
-        
-        for (my $extra  = 0; $extra < $extrachars; $extra ++) {
-            push @result, 0;
-        }
-       
-    } elsif( $typeid == OPENTHINGS_FLOAT ) {
-        warn "TODO_FLOAT_IEEE_754-2008";
-        return @emptyresult;
-        
-    } elsif($typeid >=  OPENTHINGS_SINT && $typeid <= OPENTHINGS_SINT_BP24 ) {
-		# signed int
-        if ( $typeid != OPENTHINGS_SINT ) {
-			# pre-adjust for BP
-			if( int($value) != $value ) {
-				$value *= ( 2 ** $self->get_value_type_bits( $typeid ) ); # shifts float into int range using BP
-				$value = int($value);
-            }
-        }
-        
-        $value = int($value);
-        my $negative = 0;
-        
-        if( $value < 0 ) {
-            my $bits;
-			if( $typeid == OPENTHINGS_SINT ) {
-				$bits = $self->get_value_bits($value);
-            } else {
-				$bits = $self->get_value_type_bits( $typeid );
-            }
-    
-            # NORMALISE BITS TO BYTES
-            #round up to nearest number of 8 bits
-            # if already 8, leave 1,2,3,4,5,6,7,8 = 8   0,1,2,3,4,5,6,7 (((b-1)/8)+1)*8
-            # 9,10,11,12,13,14,15,16=16
-            $bits = ((($bits-1)/8)+1)*8; # snap to nearest byte boundary
-
-            $value &= ((2**int($bits))-1);
-            $negative = 1;
-        }
-        
-        my $v = $value;
-		while( $v != 0 ) {
-            unshift(@result, $v & 0xFF );
-            $v >>= 8;
-        }
-        
-        if( $vallength && $vallength > ( scalar @result) ) {
-            # left pad with zeros
-			my $numbytes = $vallength - ( scalar @result );
-            if( $negative ) {
-                for (my $index = 0; $index < $numbytes; $index ++) {
-                    unshift(@result, 0xFF);
-                }
-            } else {
-                for (my $index = 0; $index < $numbytes; $index ++) {
-                    unshift(@result, 0);
-                }
-            }
-        }
     }
     
     return @result;
@@ -491,135 +602,24 @@ sub add_record {
     return;
 }
 
-sub ot_param_map {
-    my ($self, $param ) = @_;
-    
-    my($name, $units) = ('Unknown - ' . $param, '');
-    
-    if ( $param == OPENTHINGS_PARAM_ALARM ) {
-        $name = 'Alarm';
-    } elsif( $param == OPENTHINGS_PARAM_DEBUG_OUTPUT ) {
-        $name = 'Debug Output';
-    } elsif( $param == OPENTHINGS_PARAM_IDENTIFY ) {
-        $name = 'Identify';
-    } elsif( $param == OPENTHINGS_PARAM_SOURCE_SELECTOR ) {
-        $name = 'Source Selector';
-    } elsif( $param == OPENTHINGS_PARAM_WATER_DETECTOR ) {
-        $name = 'Water Detector';
-    } elsif( $param == OPENTHINGS_PARAM_GLASS_BREAKAGE ) {
-        $name = 'Glass Breakage';
-    } elsif( $param == OPENTHINGS_PARAM_CLOSURES ) {
-        $name = 'Closures';
-    } elsif( $param == OPENTHINGS_PARAM_DOOR_BELL ) {
-        $name = 'Door Bell';
-    } elsif( $param == OPENTHINGS_PARAM_ENERGY ) {
-        $name = 'Energy';
-        $units = 'kWh'
-    } elsif( $param == OPENTHINGS_PARAM_FALL_SENSOR ) {
-        $name = 'Fall Sensor';
-    } elsif( $param == OPENTHINGS_PARAM_GAS_VOLUME ) {
-        $name = 'Gas Volume';
-        $units = 'm3';
-    } elsif( $param == OPENTHINGS_PARAM_AIR_PRESSURE ) {
-        $name = 'Air Pressure';
-        $units = 'mbar';
-    } elsif( $param == OPENTHINGS_PARAM_ILLUMINANCE ) {
-        $name = 'Illuminance';
-        $units = 'Lux';
-    } elsif( $param == OPENTHINGS_PARAM_LEVEL ) {
-        $name = 'Level';
-    } elsif( $param == OPENTHINGS_PARAM_RAINFALL ) {
-        $name = 'Rainfall';
-        $units = 'mm';
-    } elsif( $param == OPENTHINGS_PARAM_APPARENT_POWER ) {
-        $name = 'Apparent Power';
-        $units = 'VA';
-    } elsif( $param == OPENTHINGS_PARAM_POWER_FACTOR ) {
-        $name = 'Power Factor';
-    } elsif( $param == OPENTHINGS_PARAM_REPORT_PERIOD ) {
-        $name = 'Report Period';
-        $units = 's';
-    } elsif( $param == OPENTHINGS_PARAM_SMOKE_DETECTOR ) {
-        $name = 'Smoke Detector';
-    } elsif( $param == OPENTHINGS_PARAM_TIME_AND_DATE ) {
-        $name = 'Time and Date';
-        $units = 's';
-    } elsif( $param == OPENTHINGS_PARAM_VIBRATION ) {
-        $name = 'Vibration';
-    } elsif( $param == OPENTHINGS_PARAM_WATER_VOLUME ) {
-        $name = 'Water Volume';
-        $units = 'l';
-    } elsif( $param == OPENTHINGS_PARAM_WIND_SPEED ) {
-        $name = 'Wind Speed';
-        $units = 'm/s';
-    } elsif( $param == OPENTHINGS_PARAM_GAS_PRESSURE ) {
-        $name = 'Gas Pressure';
-        $units = 'Pa';
-    } elsif( $param == OPENTHINGS_PARAM_BATTERY_LEVEL ) {
-        $name = 'Battery Level';
-        $units = 'V';
-    } elsif( $param == OPENTHINGS_PARAM_CO_DETECTOR ) {
-        $name = 'Carbon Monoxide Detector';
-    } elsif( $param == OPENTHINGS_PARAM_DOOR_SENSOR ) {
-        $name = 'Door Sensor';
-    } elsif( $param == OPENTHINGS_PARAM_EMERGENCY ) {
-        $name = 'Emergency';
-    } elsif( $param == OPENTHINGS_PARAM_FREQUENCY ) {
-        $name = 'Frequency';
-        $units = 'Hz';
-    } elsif( $param == OPENTHINGS_PARAM_GAS_FLOW_RATE ) {
-        $name = 'Gas Flow Rate';
-        $units = 'm3/hr';
-    } elsif( $param == OPENTHINGS_PARAM_RELATIVE_HUMIDITY ) {
-        $name = 'Relative Humidity';
-        $units = '%';
-    } elsif( $param == OPENTHINGS_PARAM_CURRENT ) {
-        $name = 'Current';
-        $units = 'A';
-    } elsif( $param == OPENTHINGS_PARAM_JOIN ) {
-        $name = 'Join';
-    } elsif( $param == OPENTHINGS_PARAM_LIGHT_LEVEL ) {
-        $name = 'Light Level';
-    } elsif( $param == OPENTHINGS_PARAM_MOTION_DETECTOR ) {
-        $name = 'Motion Detector';
-    } elsif( $param == OPENTHINGS_PARAM_OCCUPANCY ) {
-        $name = 'Occupancy';
-    } elsif( $param == OPENTHINGS_PARAM_REAL_POWER ) {
-        $name = 'Real Power';
-        $units = 'W';
-    } elsif( $param == OPENTHINGS_PARAM_REACTIVE_POWER ) {
-        $name = 'Reactive Power';
-        $units = 'VAR';
-    } elsif( $param == OPENTHINGS_PARAM_SWITCH_STATE ) {
-        $name = 'Switch State';
-    } elsif( $param == OPENTHINGS_PARAM_TEMPERATURE ) {
-        $name = 'Temperature';
-        $units = 'C';
-    } elsif( $param == OPENTHINGS_PARAM_VOLTAGE ) {
-        $name = 'Voltage';
-        $units = 'V';
-    } elsif( $param == OPENTHINGS_PARAM_WATER_FLOW_RATE ) {
-        $name = 'Water Flow Rate';
-        $units = 'l/hr';
-    } elsif( $param == OPENTHINGS_PARAM_WATER_PRESSURE ) {
-        $name = 'Water Pressure';
-        $units = 'Pa';
-    } elsif( $param == OPENTHINGS_PARAM_PHASE_1_POWER ) {
-        $name = 'Phase 1 Power';
-        $units = 'W';
-    } elsif( $param == OPENTHINGS_PARAM_PHASE_2_POWER ) {
-        $name = 'Phase 2 Power';
-        $units = 'W';
-    } elsif( $param == OPENTHINGS_PARAM_PHASE_3_POWER ) {
-        $name = 'Phase 3 Power';
-        $units = 'W';
-    } elsif( $param == OPENTHINGS_PARAM_3_PHASE_TOTAL ) {
-        $name = '3 Phase Total Power';
-        $units = 'W';
-    } elsif( $param == OPENTHINGS_PARAM_TEST ) {
-        $name = 'Test';
-    }
-    return ( wantarray ) ? ( $name, $units ) : $name;
+sub timestamp {
+    my($self) = @_;
+    my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = gmtime( $self->epoch );
+    my $timestamp = sprintf('%u-%02u-%02u %02u:%02u:%02u',
+        $year + 1900, $mon + 1, $mday, $hour, $min, $sec      
+    );
+    return $timestamp;
+}
+
+sub round_value {
+    my( $self, $value ) = @_;
+    if($value == 0) {
+        return 0;
+    } elsif( $value == int($value) ) {
+        return $value;
+    } else {
+        return sprintf(qq(%.0f), $value );
+    }   
 }
 
 #########################################################################################
@@ -631,14 +631,15 @@ use strict;
 use warnings;
 use parent qw( HiPi::Class );
 
-__PACKAGE__->create_accessors( qw( wr id name units typeid length value bytes ));
+__PACKAGE__->create_accessors( qw( command id name units typeid length value bytes ) );
 
 sub new {
     my ( $class, %params ) = @_;
+    
     my $self = $class->SUPER::new( %params );
     
     unless($self->name && $self->units) {
-        my ( $name, $units ) = HiPi::RF::OpenThings::Message->ot_param_map( $self->id );
+        my ( $name, $units ) = HiPi::RF::OpenThings->parameter_map( $self->id );
         unless( $self->name ) {
             $self->name( $name );
         }
